# a plugin for pympd that scrapes data from audioscrobbler.com

from HTMLParser import HTMLParser
import urllib
import gtk, gtk.glade
import pygtk
import threading


NUM_ARTISTS = 11
class ASHTMLParser(HTMLParser):
    def __init__(self, num_artists):
        HTMLParser.__init__(self)
        self.num_artists = num_artists
         
    def handle_starttag(self, tag, attrs):
        self.bar = False
        if tag == 'a':
            for attr in attrs:
                if attr[0] == 'href' and attr[1].find('/music/') > -1:
                    self.link_opened = True
        if tag =='div':
            for attr in attrs:
                if attr[0] == 'class' and attr[1] == 'bar':
                     self.bar = True
                     continue
        if self.bar:
            for attr in attrs:
                if attr[0] == 'style':
                    if not self.artist_count > self.num_artists:
                        self.similarity.extend([attr[1][attr[1].find(':')+1:]])


    def handle_endtag(self, tag):
        self.link_opened = False
    
    def handle_data(self, data):
        if not self.artist_count > self.num_artists:
            if self.link_opened:
                self.artist_count += 1
                self.artists.extend([data])

    def clean(self):
        self.artist_count = 0
        self.artists = []
        self.similarity = ['100%']
        self.link_opened = False
        self.bar = False
        
    def set_num_artists(self, num):
        self.num_artists = num

class Audioscrobbler:

    # called when we load the plugin. we get passed the mainWindow, plugin menu
    # item as well as a connection to mpd.
    def _init(self, data):
        self.mainWindow, self.pluginMenu, self.pympd, self.base_dir = data
        self.buildPluginMenu(self.pluginMenu)
        self.ASParser = ASHTMLParser(NUM_ARTISTS)
        self.buildWindows()
        self.urlfile = urllib.URLopener()
        self.ASurl = "http://www.last.fm/"
#        self.ASurl = "http://www.audioscrobbler.com/"

    # plugin configure dialog.
    def _conf(self, hasDialog=None):
        return False
#        if not hasDialog == None and hasDialog:
#            return True
#        self.confDialog.present()
#        return 
    
    # update plugin, data is of type mpdclient.Status
    def _spin(self, data, songChanged=False):
        pass

    # called on plugin unloaded
    def _unload(self):
        self.pluginMenuEntry.destroy()
        self.resultDialog.destroy()
        self.confDialog.destroy()
        return

    # callback for find similar artists in menu
    def similarArtists(self, obj):
        song = self.pympd.getCurrentSong()
        thread = threading.Thread(target=self.OpenURL, args=[song.artist])
        thread.start()

    def OpenURL(self, artist):
        gtk.threads_enter()
        self.resultDialog.show_all()
        self.label.set_label("<big><b>Retrieving\nResults...</b></big>")
        gtk.threads_leave()
        file = self.urlfile.open("%s/explore/?artistname=%s" % (self.ASurl, artist.replace(' ', '+')))
        self.ASParser.clean()
        data = file.readlines()
        for line in data:
            self.ASParser.feed(line)
        self.urlfile.close()
        self.urlfile.cleanup()
        self.listStore.clear()
        rows = zip(self.ASParser.artists, self.ASParser.similarity)
        gtk.threads_enter()
        try:
            self.label.set_label("<big><b>Similar Artists to\n<u>%s</u></b></big>"%artist)
            for n in range(1, len(rows)):
                self.listStore.append(rows[n])
            self.treeView.set_model(self.listStore)
        except:
            self.listStore.clear()
            self.label.set_label("<big><b>Error in\nRetrieving Results.</b></big>")
        gtk.threads_leave()

        
    # Init Functions
    def buildPluginMenu(self, pluginMenu):
        self.pluginMenuEntry = gtk.MenuItem('Audioscrobbler')
        pluginSubMenu = gtk.Menu()
        pluginSubMenu.set_title('Audioscrobbler')
        menuEntry = gtk.MenuItem('Show similar artists')
        menuEntry.connect('activate', self.similarArtists)
        pluginSubMenu.append(menuEntry)
        self.pluginMenuEntry.set_submenu(pluginSubMenu)
        self.pluginMenuEntry.show_all()
        pluginMenu.append(self.pluginMenuEntry)

    # Build the window we wanna display the information in. Somehow.
    def buildWindows(self):
        self.xml = gtk.glade.XML("%s/glade/audioscrobbler.glade"%self.base_dir)
        self.resultDialog = self.xml.get_widget("resultDialog")
        self.confDialog = self.xml.get_widget("confDialog")
        self.spinner = self.xml.get_widget("spinbutton1")
        self.spinner.set_value(NUM_ARTISTS-1)
        self.treeView = self.xml.get_widget("treeview1")
        self.listStore = gtk.ListStore(str, str)
        self.label = self.xml.get_widget("label1")
        colHeadings = ['Artist', 'Popularity']
        columns = [None] * 2
        cellStr = gtk.CellRendererText()
        for n in range(2):
            columns[n] = gtk.TreeViewColumn(colHeadings[n])
            self.treeView.append_column(columns[n])
            columns[n].pack_start(cellStr, True)
            columns[n].set_attributes(cellStr, text=n)
            columns[n].set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

        dic = {"on_dialog_closed" : self.close_dialog,
               "on_dialog_ok"     : self.close_dialog }
        self.xml.signal_autoconnect(dic)

    # TODO: make more of these, one for destroy, 2 for close buttons.
    def close_dialog(self, obj, data=None):
        if obj == self.resultDialog:
            self.listStore.clear()
            self.resultDialog.hide()
        elif obj == self.confDialog:
            self.num_artists = self.spinner.get_value() + 1
            self.ASParser.set_num_artists(self.num_artists)
            self.confDialog.hide()
        else:
            self.listStore.clear()
            self.resultDialog.hide()
        return True
        
